from dataclasses import dataclass

@dataclass
class InferArgs:
  """
  Data class for inference arguments.
  
  Attributes:
    max_new_tokens: Maximum number of new tokens to generate
    temperature: Temperature
    top_p: Top-P. If typical sampling isn't used, generation defaults to top-p sampling
    typical_mass: Mass parameter. If set, generation will use typical sampling
    max_rep_ctx: Maximum number of tokens to look back for repetition penalty
    rep_penalty: Repetition penalty, applied to logits for every token
  """
  max_new_tokens: int = 512
  temperature: float = 0.0
  top_p: float = 0.0
  typical_mass: float = 0.0
  max_rep_ctx: int = 64
  rep_penalty: float = 1.0